==> evmspec-0.0.4.tar.gz: evmspec-0.0.4/evmspec/receipt.py <==
from decimal import Decimal
from enum import Enum, EnumMeta
from functools import cached_property
from typing import Optional, Tuple

from dictstruct import DictStruct, LazyDictStruct
from hexbytes import HexBytes
from msgspec import UNSET, Raw, field, json

from evmspec._ids import TransactionIndex
from evmspec.data import Address, BlockNumber, TransactionHash, Wei, uint, _decode_hook
from evmspec.log import Log


class FeeStats(DictStruct, frozen=True, forbid_unknown_fields=True):  # type: ignore [call-arg]
    """Arbitrum includes this in the `feeStats` field of a tx receipt."""

    l1Calldata: Wei
    l2Storage: Wei
    l1Transaction: Wei
    l2Computation: Wei


class ArbitrumFeeStats(DictStruct, frozen=True, forbid_unknown_fields=True, omit_defaults=True, repr_omit_defaults=True):  # type: ignore [call-arg]
    """Arbitrum includes these with a tx receipt."""

    paid: FeeStats
    """
    The breakdown of gas paid for the transaction.
    
    (price * unitsUsed)
    """
    # These 2 attributes do not always exist
    unitsUsed: FeeStats = UNSET  # type: ignore [assignment]
    """The breakdown of units of gas used for the transaction."""
    prices: FeeStats = UNSET  # type: ignore [assignment]
    """The breakdown of gas prices for the transaction."""


class _HexStringToIntEnumMeta(EnumMeta):
    """
    A metaclass for Enums that allows conversion from hexadecimal string values to integer Enum members.
    """

    def __call__(cls, value: str, *args, **kw):
        return super().__call__(int(value, 16), *args, **kw)


class Status(Enum, metaclass=_HexStringToIntEnumMeta):
    """
    Enum representing the status of a transaction, indicating success or failure.
    """

    failure = 0
    success = 1


class TransactionReceipt(LazyDictStruct, frozen=True, kw_only=True, omit_defaults=True, repr_omit_defaults=True, dict=True):  # type: ignore [call-arg]

    transactionHash: TransactionHash
    """The unique hash of this transaction."""

    blockNumber: BlockNumber
    """The block number that contains the transaction."""

    contractAddress: Optional[Address]
    """The contract address created, if the transaction was a contract creation, otherwise `None`."""

    transactionIndex: TransactionIndex
    """The position of this transaction within the block."""

    status: Status
    """1 if the transaction succeeded, 0 if it failed."""

    gasUsed: Wei
    """The amount of gas used by this transaction, not counting internal transactions, calls or delegate calls."""

    cumulativeGasUsed: Wei
    """The total amount of gas used during this transaction."""

    _logs: Raw = field(name="logs")
    """The logs that were generated during this transaction."""

    @cached_property
    def logs(self) -> Tuple[Log, ...]:
        """The logs that were generated during this transaction."""
        return json.decode(self._logs, type=Tuple[Log, ...], dec_hook=_decode_hook)

    # These fields are only present on Mainnet.
    effectiveGasPrice: Wei = UNSET  # type: ignore [assignment]
    """
    The actual value per gas deducted from the sender's account.
    
    This field is only present on Mainnet.
    """

    type: uint = UNSET  # type: ignore [assignment]
    """
    The transaction type.

    This field is only present on Mainnet.
    """

    blobGasUsed: Wei = UNSET  # type: ignore [assignment]
    """This field is sometimes present, only on Mainnet."""

    # These fields are only present on Optimism.
    l1FeeScalar: Decimal = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""
    l1GasUsed: Wei = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""
    l1GasPrice: Wei = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""
    l1Fee: Wei = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""

    # These fields are only present on Arbitrum.
    l1BlockNumber: BlockNumber = UNSET  # type: ignore [assignment]
    """This field is only present on Arbitrum."""
    l1InboxBatchInfo: Optional[HexBytes] = UNSET  # type: ignore [assignment]
    """This field is only present on Arbitrum."""
    _feeStats: Raw = field(name="feeStats", default=UNSET)  # type: ignore [assignment]
    """This field is only present on Arbitrum."""

    @cached_property
    def feeStats(self) -> ArbitrumFeeStats:
        """This field is only present on Arbitrum."""
        return json.decode(
            self._feeStats, type=ArbitrumFeeStats, dec_hook=Wei._decode_hook
        )


class FullTransactionReceipt(TransactionReceipt, frozen=True, kw_only=True, forbid_unknown_fields=True, omit_defaults=True, repr_omit_defaults=True):  # type: ignore [call-arg]

    blockHash: HexBytes
    """The hash of the block that contains the transaction."""

    logsBloom: HexBytes
    """The bloom filter for all logs in this block."""

==> evmspec-0.0.5.tar.gz: evmspec-0.0.5/evmspec/receipt.py <==
from decimal import Decimal
from enum import Enum, EnumMeta
from functools import cached_property
from typing import Optional, Tuple

from dictstruct import DictStruct, LazyDictStruct
from hexbytes import HexBytes
from msgspec import UNSET, Raw, field, json

from evmspec._ids import TransactionIndex
from evmspec.data import Address, BlockNumber, TransactionHash, Wei, uint, _decode_hook
from evmspec.log import Log


class FeeStats(DictStruct, frozen=True, forbid_unknown_fields=True):  # type: ignore [call-arg]
    """Arbitrum includes this in the `feeStats` field of a tx receipt."""

    l1Calldata: Wei
    l2Storage: Wei
    l1Transaction: Wei
    l2Computation: Wei


class ArbitrumFeeStats(DictStruct, frozen=True, forbid_unknown_fields=True, omit_defaults=True, repr_omit_defaults=True):  # type: ignore [call-arg]
    """Arbitrum includes these with a tx receipt."""

    paid: FeeStats
    """
    The breakdown of gas paid for the transaction.

    (price * unitsUsed)
    """
    # These 2 attributes do not always exist
    unitsUsed: FeeStats = UNSET  # type: ignore [assignment]
    """The breakdown of units of gas used for the transaction."""
    prices: FeeStats = UNSET  # type: ignore [assignment]
    """The breakdown of gas prices for the transaction."""


class _HexStringToIntEnumMeta(EnumMeta):
    """
    A metaclass for Enums that allows conversion from hexadecimal string values to integer Enum members.
    """

    def __call__(cls, value: str, *args, **kw):
        return super().__call__(int(value, 16), *args, **kw)


class Status(Enum, metaclass=_HexStringToIntEnumMeta):
    """
    Enum representing the status of a transaction, indicating success or failure.
    """

    failure = 0
    success = 1


class TransactionReceipt(LazyDictStruct, frozen=True, kw_only=True, omit_defaults=True, repr_omit_defaults=True, dict=True):  # type: ignore [call-arg]
    """Represents the receipt of a transaction within a block."""

    transactionHash: TransactionHash
    """The unique hash of this transaction."""

    blockNumber: BlockNumber
    """The block number that contains the transaction."""

    contractAddress: Optional[Address]
    """The contract address created, if the transaction was a contract creation, otherwise `None`."""

    transactionIndex: TransactionIndex
    """The position of this transaction within the block."""

    status: Status
    """1 if the transaction succeeded, 0 if it failed."""

    gasUsed: Wei
    """The amount of gas used by this transaction, not counting internal transactions, calls or delegate calls."""

    cumulativeGasUsed: Wei
    """The total amount of gas used in the block up to and including this transaction."""

    _logs: Raw = field(name="logs")
    """The logs that were generated during this transaction."""

    @cached_property
    def logs(self) -> Tuple[Log, ...]:
        """The logs that were generated during this transaction."""
        return json.decode(self._logs, type=Tuple[Log, ...], dec_hook=_decode_hook)

    # These fields are only present on Mainnet.
    effectiveGasPrice: Wei = UNSET  # type: ignore [assignment]
    """
    The actual value per gas deducted from the sender's account.

    This field is only present on Mainnet.
    """

    type: uint = UNSET  # type: ignore [assignment]
    """
    The transaction type.

    This field is only present on Mainnet.
    """

    blobGasUsed: Wei = UNSET  # type: ignore [assignment]
    """This field is sometimes present, only on Mainnet."""

    # These fields are only present on Optimism.
    l1FeeScalar: Decimal = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""
    l1GasUsed: Wei = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""
    l1GasPrice: Wei = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""
    l1Fee: Wei = UNSET  # type: ignore [assignment]
    """This field is only present on Optimism."""

    # These fields are only present on Arbitrum.
    l1BlockNumber: BlockNumber = UNSET  # type: ignore [assignment]
    """This field is only present on Arbitrum."""
    l1InboxBatchInfo: Optional[HexBytes] = UNSET  # type: ignore [assignment]
    """This field is only present on Arbitrum."""
    _feeStats: Raw = field(name="feeStats", default=UNSET)  # type: ignore [assignment]
    """This field is only present on Arbitrum."""

    @cached_property
    def feeStats(self) -> ArbitrumFeeStats:
        """This field is only present on Arbitrum."""
        return json.decode(
            self._feeStats, type=ArbitrumFeeStats, dec_hook=Wei._decode_hook
        )


class FullTransactionReceipt(TransactionReceipt, frozen=True, kw_only=True, forbid_unknown_fields=True, omit_defaults=True, repr_omit_defaults=True):  # type: ignore [call-arg]
    """Extends TransactionReceipt to include full details."""

    blockHash: HexBytes
    """The hash of the block that contains the transaction."""

    logsBloom: HexBytes
    """The bloom filter for all logs in this block."""
